// DE_method.h: interface for the CDE_method class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DE_METHOD_H__5820314D_6249_411C_9801_41F175DA39AF__INCLUDED_)
#define AFX_DE_METHOD_H__5820314D_6249_411C_9801_41F175DA39AF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "global.h"
#include "./EACommonClass/Individual.h"
#include "./EACommonClass/Rand.h"
#include "./EACommonClass/ConstrainedProblemDef.h"

#include <direct.h>				// include the mkdir() for folder creation

#define out_internal    1		// Ƶ

/* Ӳ̣ԲҪһֽһֽڸ */
#define SIZEOFBUFFER	256*1024L /* СĬΪ256KB */


class CDE_method
{
public:
	CDE_method();
	virtual ~CDE_method();

	/* ------------------------class methods------------------------ */
public:
	// public methods
	/* initialize the decision variables of different test problems */
	void init_variables();							// initialize the decision variables
	void init_constrained_variables();				// initialize the constrained functions' variables

	/* initialize the population */
	void init_pop();								// initialize the population
	void init_pop_random();							// initialize the population randomly

	/* evaluate the individual and population */
	void evaluate_ind(CIndividual &);				// evaluate the individual
	void evaluate_constrained_ind(CIndividual &);	// evaluate the individual of constrained functions

	void evaluate_pop(CIndividual *, int);			// evaluate the population

	/* execute the algorithms */
	void Run_Optimizer(int, int, double, int, int);	// execute the optimizer

	/* compare the two individuals */
	int  compare_ind(CIndividual *, CIndividual *);	// compare the two individuals
	void find_best_index(CIndividual *, int);		// find the index of the best individual in the population

	void restart_pop(CIndividual *, int);			// restart the population
	void sort_pop(CIndividual *, int);				// sort the population w.r.t fitness with ascend order

	/* some complementary methods */
	void random_index(int *array_index, int all_size, int size);

	void shell_sort_pop(CIndividual* ,int *, int);		// shell sort routine in ascending order
	void shell_sort_array(double *, int *, int);
	void shell_sort_array_t(tFitness *, int *, int);

	inline  int round(double x) { return int(x>0.0?(x+0.5):(x-0.5)); } // round function
	double  modulo(double a, double b);

	double median(double *array, int size);			// get the median value of the specific array
	double mean_std(double *array, int size, double &stdv);	// get the mean and standard values of the specific array
	tFitness mean_std_t(tFitness *array, int size, tFitness &stdv);
	void   statistic_pop_fitness(CIndividual *pop, int size);

	void display_result(long int);						// display the results in the screen
	void report_result(long int, ofstream &);			// report the results in the files
	void report_diversity(long int, ofstream &);			// report the diversity in the files
	void report_parameter(long int gen, ofstream &file);

	/************************************************************************/
	/* Methods and attributes for batch execution (ִг)      */
	/************************************************************************/
public:
	void	create_folders(int method_ii, int strategy_ii);		// create the folders
	void	copy_files(char *source, char *target);				// copy files from the source directory to the target directory
	long	filesize(FILE *stream);
	int		copyfile(const char* src,const char* dest);

	/* ------------------------class members------------------------ */
public:
	// public attributes
	CIndividual *parent_pop;				// the main population
	CIndividual *child_pop;					// the child population
	CIndividual *child_pop1;				// the child population used in BBO_DE
	CIndividual best_individual;			// the current best individual

	CIndividual	mutant_vector;				// save the mutant vector
	CIndividual new_child;					// the new child generated by the genetic operator

	int pop_size;							// size of the main population
	int best_index;							// index of the current best individual
	int worst_index;

	int flag_precision;						// stop criterion

	long int max_NFFEs;						// maximal number of fitness function evaluations
	long int max_iterations;				// maximal number of iterations
	long int evaluations;					// current number of fitness function evaluations
	long int gen;							// current generation counter
	int MINIMIZE;

	double *upper_bounds;					// upper bounds of the decision variables
	double *lower_bounds;					// lower bounds of the decision variables
	double known_optimal;					// known optima of the problems

	double pop_diversity;
	double max_interval;

	/* some other class objectives */
	double rnd_seed;
	CRand  m_rnd;

	int func_flag;								// flag of the function optimized
	CConstrainedProblemDef	constrained_func;	// constrained test functions

	int method_flag;

	int	*sort_index;							// only for sorting the population

	/************************************************************************/
	/* Methods and attributes for MDDE algorithm                            */
	/************************************************************************/
public:
	double	m_F;															// scale factor
	double	m_CR;															// probability of DE crossover
	int		m_strategy;														// mutation strategy

	void	DE_rand_1_exp(CIndividual &indv, int i, int r1, int r2, int r3, double cr, double ff);
	void	DE_rand_1_bin(CIndividual &indv, int i, int r1, int r2, int r3, double cr, double ff);

	int		no_of_children;													// number of children generated by each parent
	double	SR;																// the probability to accept the infeasible individual
	double	SR_0;

	int		SR_selection(CIndividual &, CIndividual &);						// stochastic ranking selection
	void	shell_sort_pop_constraints(CIndividual* ,int *, int);			// shell sort routine in ascending order with constraints
	void	SR_bubble_sort(CIndividual* ,int *, int, double);
	void	SR_shell_sort_pop_constraints(CIndividual* ,int *, int, double);	// shell sort routine in ascending order with constraints

	/* Ref:
	   1. Multiple trial vectors in differential evolution for engineering design
	   2. Promising Infeasibility and Multiple Offspring Incorporated to Differential Evolution for Constrained Optimization
	*/
	void	run_MDDE_COPs();						// the original MDDE method
	void	run_rank_iMDDE_COPs();				// our proposal: MDDE + adaptive ranking-based mutation operators + dynamic stochastic ranking

	/* Ranking-based on the feasible status */
	/* Ref:
	   Y. Wang, Z. Cai,
	   Constrained evolutionary optimization by means of mu+lambda-differential evolution and improved adaptive trade-off model,
	   Evolutionary Computation, 2011 19(2): 249-285
	*/
	void	feasibility_adaptive_ranking_probability(CIndividual *pop, int size, double *probability);
	void	semifeasible_fitness_transformation(CIndividual *pop, int size, double *transformed_fitness, double current_feasible_ratio);
};

#endif // !defined(AFX_DE_METHOD_H__5820314D_6249_411C_9801_41F175DA39AF__INCLUDED_)
